#include <Kernel.hpp>
#include <gdt/Gdt.hpp>
#include <idt/Idt.hpp>
#include <bios/Video.hpp>
#include <idt/InterruptRoutines.hpp>
#include <idt/InterruptRequests.hpp>

#include <system/System.hpp>
#include <system/handler/Keyboard.hpp>
#include <system/handler/Timer.hpp>

/**
 * OS main method
 */
extern "C" void createKernel(multiboot_info_t* multiboot_info,
		unsigned int magicNumber) {
	Video cout;
	cout.clear();
	cout.setColor(0x0F);

	if (magicNumber == 0x2BADB002) {
		Gdt gdt;
		gdt.install();

		Idt idt;
		idt.install();

		InterruptRoutines routines;
		routines.install(idt);

		InterruptRequests requests;
		requests.install(idt);

		Timer timer;
		timer.install(requests);

		Keyboard keyboard;
		keyboard.install(requests);

		//enable_paging(multiboot_info);

		asm volatile ("sti");

		Kernel kernel(multiboot_info);
		kernel.run();
	} else {
		const char grub_failed[] = "GRUB failed";
		cout.print(grub_failed);
	}
}

/**
 * Kernel constructor
 */
Kernel::Kernel(multiboot_info_t* mbd) {
	this->info = mbd;
}

/**
 * Kernel run method
 */
void Kernel::run() {
	Video cout;
	Keyboard keyboard;

	cout.clear();

	cout.printChar('>');
	cout.printChar('_');
	cout.move(-1);

	int pos = 0;
	while (true) {
		char c = keyboard.readKey();

		if (c == '\b') {
			if (pos > 0) {
				cout.printChar(' ');
				cout.move(-2);
				pos--;
			}
		} else if (c == '\n') {
			cout.printChar(' ');
			pos = 0;
			cout.printChar('\n');
			cout.printChar('>');
		} else {
			cout.printChar(c);
			pos++;
		}
		cout.printChar('_');
		cout.move(-1);
	}
}
