#include "bios/Video.hpp"

/**
 * Standard constructor setting the video memory address
 */
Video::Video() {
	videoMemory = (unsigned char *) 0xb8000;
	cursorOffset = 0;
	printColor = 0x07;
}

/**
 * Clears the entire screen
 */
void Video::clear() {
	cursorOffset = 0;
	for (int line = 0; line < CONSOLE_HEIGHT; line++) {
		for (int pos = 0; pos < CONSOLE_WIDTH; pos++) {
			videoMemory[cursorOffset++] = 0;
			videoMemory[cursorOffset++] = 0x00;
		}
	}
	cursorOffset = 0;
}

/**
 * Moves the cursor to the selected offset
 */
void Video::move(int newOffset) {
	cursorOffset += (newOffset * 2);
	if (cursorOffset < 0) {
		cursorOffset = 0;
	}
}

/**
 * Moves the cursor to the specified position
 */
void Video::setPosition(int x, int y) {
	cursorOffset = 0;
	for (; y > 0; --y) {
		cursorOffset += CONSOLE_WIDTH;
	}
	for (; x > 0; --x) {
		cursorOffset += 2;
	}
}

/**
 * Returns the current cursor offset
 */
int Video::getOffset() {
	return cursorOffset;
}

/**
 * Sets the current color
 */
void Video::setColor(int newColor) {
	printColor = newColor;
}

/**
 * Prints the given character
 */
void Video::printChar(char c) {
	if (c == '\n') {
		while (++cursorOffset % CONSOLE_WIDTH != 0) {
		}
	} else {
		videoMemory[cursorOffset++] = c;
		videoMemory[cursorOffset++] = printColor;
	}
	if (cursorOffset > CONSOLE_WIDTH * CONSOLE_HEIGHT) {
		clear();
		cursorOffset = 0;
	}
}

/**
 * Prints the given string
 */
void Video::print(const char *text) {
	int i = 0;
	while (text[i] != 0) {
		printChar(text[i++]);
	}
}
