unsigned int *pageDirectory;
unsigned int table_index = 0;

/*
 * Die erste Addresse des Upper Memory.
 * Hier wird das Page Directory hingesetzt und nach diesem der Arbeitsspeicher gemappt.
 */
unsigned int nextAddress = 0x1000000;

unsigned int USED_FLAG = 0x200;

/**
 * Legt das Page Directory nach dem Kernel im Arbeitsspeicher an.
 */
void makePageDirectory() {
        // Page-Directory auf Ende des Kernels mit 0xFFFFF000 maskiert und 0x1000 addiert
        pageDirectory = (unsigned int*) (nextAddress & 0xFFFFF000 + 0x1000);

        // Alle Eintrge auf nicht present setzen
        for (int i = 0; i < 1024; i++) {
                pageDirectory[i] = 0 | 2;
        }
}

/**
 * Erstellt einen Page Table und gibt seine Adresse zurck.
 *
 * Der Page Table enthlt Adressen, die von mappable_address angefangen immer um 4096
 * hochgezhlt werden und so den verfgbaren Speicher verwenden.
 */
void createPageTable(unsigned int* target) {
        // Addressen mappen
        for (int i = 0; i < 1024; i++) {
                target[i] = nextAddress | 3;
                nextAddress = nextAddress + 4096;
        }

        // Table ins Directory fllen und Berechtigungen setzen
        pageDirectory[table_index] = (unsigned int) target;
        pageDirectory[table_index] |= (1 | 2 | USED_FLAG); // rw, present, USED_FLAG
        table_index++;
}

/**
 * Sucht nach einem freien Table, oder
 * legt einen neuen an falls keiner gefunden wird.
 */
unsigned int* getPageTable() {
        for (int i = 0; i < 1024; i++) {
                if (pageDirectory[i] & USED_FLAG != USED_FLAG) {
                        return (unsigned int*) pageDirectory[i];
                }
        }
        return 0;
}

/**
 * Entfernt die USED_FLAG von einem Table
 */
void freeTable(unsigned int* table) {
        for (int i = 0; i < 1024; i++) {
                if (pageDirectory[i] & 0xFFFFF000 == (unsigned int) table) {
                        pageDirectory[i] ^= USED_FLAG;
                }
        }
}

/**
 * Setzt das Paging Directory in das CR3-Register und aktiviert Paging mit dem CR0-Register.
 */
void enable_paging(multiboot_info_t* info) {
        makePageDirectory();

        // Die anzahl der verfgbaren Maps
        int availableMaps = info->mmap_length / sizeof(memory_map);

        unsigned int* currentTable = 0;
        int currentPage = 0;

        memory_map_t* map = (memory_map_t*) info->mmap_addr;
        while ((unsigned long int) map < info->mmap_addr + info->mmap_length) {
                map = (memory_map_t*) ((unsigned int) map + map->size
                                + sizeof(unsigned int));

                if (map->type == 1) {
                        // So viele Tables oder Pages knnen hier eingefgt werden
/*
                        int spaceBlocks = (map->length_high << 32 + map->length_low) / 4096;
                        unsigned long address =  (map->base_addr_high << 32) + (map->base_addr_low);

                        for (int j = 0; j < spaceBlocks; j++) {
                                if (currentTable == 0) {
                                        currentTable = (unsigned int*) nextAddress;
                                } else if (currentPage == 4096) {
                                        // Table ins Directory fllen und Berechtigungen setzen
                                        pageDirectory[table_index] = (unsigned int) currentTable;
                                        pageDirectory[table_index] |= (1 | 2 | USED_FLAG); // rw, present, USED_FLAG
                                        table_index++;

                                        currentPage = 0;
                                        currentTable = 0;
                                } else {
                                        currentTable[currentPage] = nextAddress | 3;
                                        currentPage++;
                                }

                                nextAddress += 4096;
                        }*/
                }
        }

        // Aktivieren TODO
        asm volatile("mov %0, %%cr3":: "b"(pageDirectory));
        unsigned int cr0;
        asm volatile("mov %%cr0, %0": "=b"(cr0));
        cr0 |= 0x80000000;
        asm volatile("mov %0, %%cr0":: "b"(cr0));
}
